@@ -2,13 +2,13 @@ import os
 
 from StringIO import StringIO
 
-from twisted.trial import unittest
-
 from klein import Klein
 
 from klein.interfaces import IKleinRequest
 from klein.resource import KleinResource, ensure_utf8_bytes
 
+from klein.test_util import TestCase
+
 from twisted.internet.defer import succeed, Deferred, fail, CancelledError
 from twisted.internet.error import ConnectionLost
 from twisted.web import server
@@ -48,23 +48,55 @@ def requestMock(path, method="GET", host="localhost", port=8080, isSecure=False,
     request.setHeader = Mock(wraps=request.setHeader)
     request.setResponseCode = Mock(wraps=request.setResponseCode)
 
-    request.finish = Mock(wraps=request.finish)
-    request.write = Mock(wraps=request.write)
+    request._written = StringIO()
+    request.finishCount = 0
+    request.writeCount = 0
 
     def registerProducer(producer, streaming):
-        # This is a terrible terrible hack.
-        producer.resumeProducing()
-        producer.resumeProducing()
+        request.producer = producer
+        for x in range(2):
+            if request.producer:
+                request.producer.resumeProducing()
+
+    def unregisterProducer():
+        request.producer = None
+
+    def finish():
+        request.finishCount += 1
+
+        if not request.startedWriting:
+            request.write('')
+
+        if not request.finished:
+            request.finished = True
+            request._cleanup()
+
+    def write(data):
+        request.writeCount += 1
+        request.startedWriting = True
+
+        if not request.finished:
+            request._written.write(data)
+        else:
+            raise RuntimeError('Request.write called on a request after '
+                'Request.finish was called.')
+
+    def getWrittenData():
+        return request._written.getvalue()
+
+    request.finish = finish
+    request.write = write
+    request.getWrittenData = getWrittenData
 
     request.registerProducer = registerProducer
-    request.unregisterProducer = Mock()
+    request.unregisterProducer = unregisterProducer
 
     request.processingFailed = Mock(wraps=request.processingFailed)
 
     return request
 
 
-def _render(resource, request):
+def _render(resource, request, notifyFinish=True):
     result = resource.render(request)
 
     if isinstance(result, str):
@@ -72,7 +104,7 @@ def _render(resource, request):
         request.finish()
         return succeed(None)
     elif result is server.NOT_DONE_YET:
-        if request.finished:
+        if request.finished or not notifyFinish:
             return succeed(None)
         else:
             return request.notifyFinish()
@@ -121,13 +153,57 @@ class ChildrenResource(Resource):
         return ChildResource(path)
 
 
+class ProducingResource(Resource):
+    def __init__(self, path, strings):
+        self.path = path
+        self.strings = strings
 
-class KleinResourceTests(unittest.TestCase):
+    def render_GET(self, request):
+        producer = MockProducer(request, self.strings)
+        producer.start()
+        return server.NOT_DONE_YET
+
+
+class MockProducer(object):
+    def __init__(self, request, strings):
+        self.request = request
+        self.strings = strings
+
+    def start(self):
+        self.request.registerProducer(self, False)
+
+    def resumeProducing(self):
+        if self.strings:
+            self.request.write(self.strings.pop(0))
+        else:
+            self.request.unregisterProducer()
+            self.request.finish()
+
+
+class KleinResourceTests(TestCase):
     def setUp(self):
         self.app = Klein()
         self.kr = KleinResource(self.app)
 
 
+    def assertFired(self, deferred, result=None):
+        """
+        Assert that the given deferred has fired with the given result.
+        """
+        self.assertEqual(self.successResultOf(deferred), result)
+
+
+    def assertNotFired(self, deferred):
+        """
+        Assert that the given deferred has not fired with a result.
+        """
+        _pawn = object()
+        result = getattr(deferred, 'result', _pawn)
+        if result != _pawn:
+            self.fail("Expected deferred not to have fired, "
+                      "but it has: %r" % (deferred,))
+
+
     def test_simplePost(self):
         app = self.app
 
@@ -148,18 +224,12 @@ class KleinResourceTests(unittest.TestCase):
         request2 = requestMock('/')
 
         d = _render(self.kr, request)
-
-        def _cb(result):
-            request.write.assert_called_with('posted')
-            return _render(self.kr, request2)
-
-        d.addCallback(_cb)
-
-        def _cb2(result):
-            request2.write.assert_called_with('gotted')
-
-        d.addCallback(_cb2)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'posted')
+        
+        d2 = _render(self.kr, request2)
+        self.assertFired(d2)
+        self.assertEqual(request2.getWrittenData(), 'gotted')
 
 
     def test_simpleRouting(self):
@@ -173,12 +243,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with('ok')
-
-        d.addCallback(_cb)
-
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'ok')
 
 
     def test_branchRendering(self):
@@ -192,12 +258,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_once_with('ok')
-
-        d.addCallback(_cb)
-
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'ok')
 
 
     def test_branchWithExplicitChildrenRouting(self):
@@ -216,18 +278,13 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with('zeus')
-            return _render(self.kr, request2)
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'zeus')
+        
+        d2 = _render(self.kr, request2)
 
-        d.addCallback(_cb)
-
-        def _cb2(result):
-            request2.write.assert_called_with('ok')
-
-        d.addCallback(_cb2)
-
-        return d
+        self.assertFired(d2)
+        self.assertEqual(request2.getWrittenData(), 'ok')
 
 
     def test_branchWithExplicitChildBranch(self):
@@ -246,18 +303,13 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with('zeus')
-            return _render(self.kr, request2)
-
-        d.addCallback(_cb)
-
-        def _cb2(result):
-            request2.write.assert_called_with('ok')
-
-        d.addCallback(_cb2)
-
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'zeus')
+        
+        d2 = _render(self.kr, request2)
+        
+        self.assertFired(d2)
+        self.assertEqual(request2.getWrittenData(), 'ok')
 
 
     def test_deferredRendering(self):
@@ -273,13 +325,12 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with('ok')
+        self.assertNotFired(d)
 
-        d.addCallback(_cb)
         deferredResponse.callback('ok')
-
-        return d
+        
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 'ok')
 
 
     def test_elementRendering(self):
@@ -293,12 +344,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with("<h1>foo</h1>")
-
-        d.addCallback(_cb)
-
-        return d
+        self.assertFired(d)        
+        self.assertEqual(request.getWrittenData(), "<h1>foo</h1>")
 
 
     def test_leafResourceRendering(self):
@@ -312,13 +359,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with("I am a leaf in the wind.")
-
-        d.addCallback(_cb)
-
-        return d
-
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(),
+                "I am a leaf in the wind.")
 
     def test_childResourceRendering(self):
         app = self.app
@@ -330,13 +373,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with("I'm a child named betty!")
-
-        d.addCallback(_cb)
-
-        return d
-
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(),
+                "I'm a child named betty!")
 
     def test_childrenResourceRendering(self):
         app = self.app
@@ -349,12 +388,40 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with("I have children!")
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), "I have children!")
 
-        d.addCallback(_cb)
 
-        return d
+    def test_producerResourceRendering(self):
+        """
+        Test that Klein will correctly handle producing L{Resource}s.
+
+        Producing Resources close the connection by themselves, sometimes after
+        Klein has 'finished'. This test lets Klein finish its handling of the
+        request before doing more producing.
+        """
+        app = self.app
+
+        request = requestMock("/resource")
+
+        @app.route("/resource", branch=True)
+        def producer(request):
+            return ProducingResource(request, ["a", "b", "c", "d"])
+
+        d = _render(self.kr, request, notifyFinish=False)
+
+        self.assertNotEqual(request.getWrittenData(), "abcd", "The full "
+                            "response should not have been written at this "
+                            "point.")
+
+        while request.producer:
+            request.producer.resumeProducing()
+
+        self.assertEqual(self.successResultOf(d), None)
+        self.assertEqual(request.getWrittenData(), "abcd")
+        self.assertEqual(request.writeCount, 4)
+        self.assertEqual(request.finishCount, 1)
+        self.assertEqual(request.producer, None)
 
 
     def test_notFound(self):
@@ -362,13 +429,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.setResponseCode.assert_called_with(404)
-            self.assertIn("404 Not Found",
-                request.write.mock_calls[0][1][0])
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        request.setResponseCode.assert_called_with(404)
+        self.assertIn("404 Not Found", request.getWrittenData())
 
 
     def test_renderUnicode(self):
@@ -382,11 +445,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_with("\xE2\x98\x83")
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), "\xE2\x98\x83")
 
 
     def test_renderNone(self):
@@ -400,12 +460,10 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_once_with('')
-            request.finish.assert_called_once_with()
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), '')
+        self.assertEqual(request.finishCount, 1)
+        self.assertEqual(request.writeCount, 1)
 
 
     def test_staticRoot(self):
@@ -418,15 +476,12 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_once_with(
-                open(
-                    os.path.join(
-                        os.path.dirname(__file__), "__init__.py")).read())
-            request.finish.assert_called_once_with()
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 
+            open(
+                os.path.join(
+                    os.path.dirname(__file__), "__init__.py")).read())
+        self.assertEqual(request.finishCount, 1)
 
 
     def test_explicitStaticBranch(self):
@@ -440,15 +495,13 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            request.write.assert_called_once_with(
-                open(
-                    os.path.join(
-                        os.path.dirname(__file__), "__init__.py")).read())
-            request.finish.assert_called_once_with()
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.getWrittenData(), 
+            open(
+                os.path.join(
+                    os.path.dirname(__file__), "__init__.py")).read())
+        self.assertEqual(request.writeCount, 1)
+        self.assertEqual(request.finishCount, 1)
 
     def test_staticDirlist(self):
         app = self.app
@@ -461,14 +514,10 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.write.call_count, 1)
-            [c] = request.write.mock_calls
-            self.assertIn('Directory listing', c[1][0])
-            request.finish.assert_called_once_with()
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertIn('Directory listing', request.getWrittenData())
+        self.assertEqual(request.writeCount, 1)
+        self.assertEqual(request.finishCount, 1)
 
     def test_addSlash(self):
         app = self.app
@@ -480,15 +529,12 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.setHeader.call_count, 3)
-            request.setHeader.assert_has_calls(
-                [call('Content-Type', 'text/html; charset=utf-8'),
-                 call('Content-Length', '259'),
-                 call('Location', 'http://localhost:8080/foo/')])
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.setHeader.call_count, 3)
+        request.setHeader.assert_has_calls(
+            [call('Content-Type', 'text/html; charset=utf-8'),
+             call('Content-Length', '259'),
+             call('Location', 'http://localhost:8080/foo/')])
 
     def test_methodNotAllowed(self):
         app = self.app
@@ -500,11 +546,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.code, 405)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.code, 405)
 
     def test_methodNotAllowedWithRootCollection(self):
         app = self.app
@@ -520,11 +563,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.code, 405)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.code, 405)
 
     def test_noImplicitBranch(self):
         app = self.app
@@ -536,11 +576,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.code, 404)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.code, 404)
 
     def test_strictSlashes(self):
         app = self.app
@@ -555,13 +592,11 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(str(request_url[0]), "http://localhost:8080/foo/bar")
-            request.write.assert_called_with('foo')
-            self.assertEqual(request.code, 200)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(str(request_url[0]),
+            "http://localhost:8080/foo/bar")
+        self.assertEqual(request.getWrittenData(), 'foo')
+        self.assertEqual(request.code, 200)
 
     def test_URLPath(self):
         app = self.app
@@ -576,11 +611,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(str(request_url[0]), 'http://localhost:8080/egg/chicken')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(str(request_url[0]),
+            'http://localhost:8080/egg/chicken')
 
     def test_URLPath_root(self):
         app = self.app
@@ -594,11 +627,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(str(request_url[0]), 'http://localhost:8080/')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(str(request_url[0]), 'http://localhost:8080/')
 
     def test_URLPath_traversedResource(self):
         app = self.app
@@ -619,11 +649,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(str(request_url[0]), 'http://localhost:8080/resource/foo')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(str(request_url[0]),
+            'http://localhost:8080/resource/foo')
 
     def test_handlerRaises(self):
         app = self.app
@@ -644,13 +672,10 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.code, 500)
-            request.processingFailed.assert_called_once_with(failures[0])
-            self.flushLoggedErrors(RouteFailureTest)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.code, 500)
+        request.processingFailed.assert_called_once_with(failures[0])
+        self.flushLoggedErrors(RouteFailureTest)
 
     def test_genericErrorHandler(self):
         app = self.app
@@ -673,12 +698,9 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.code, 501)
-            assert not request.processingFailed.called
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.code, 501)
+        assert not request.processingFailed.called
 
     def test_typeSpecificErrorHandlers(self):
         app = self.app
@@ -697,6 +719,7 @@ class KleinResourceTests(unittest.TestCase):
 
         @app.handle_errors(TypeError)
         def handle_type_error(request, failure):
+            global type_error_handled
             type_error_handled = True
             return
 
@@ -708,20 +731,18 @@ class KleinResourceTests(unittest.TestCase):
 
         @app.handle_errors
         def handle_generic_error(request, failure):
+            global generic_error_handled
             generic_error_handled = True
             return
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.processingFailed.called, False)
-            self.assertEqual(type_error_handled, False)
-            self.assertEqual(generic_error_handled, False)
-            self.assertEqual(len(failures), 1)
-            self.assertEqual(request.code, 501)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.processingFailed.called, False)
+        self.assertEqual(type_error_handled, False)
+        self.assertEqual(generic_error_handled, False)
+        self.assertEqual(len(failures), 1)
+        self.assertEqual(request.code, 501)
 
     def test_notFoundException(self):
         app = self.app
@@ -739,19 +760,18 @@ class KleinResourceTests(unittest.TestCase):
 
         @app.handle_errors
         def handle_generic_error(request, failure):
+            global generic_error_handled
             generic_error_handled = True
             return
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.processingFailed.called, False)
-            self.assertEqual(generic_error_handled, False)
-            self.assertEqual(request.code, 404)
-            request.write.assert_called_once_with('Custom Not Found')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(request.processingFailed.called, False)
+        self.assertEqual(generic_error_handled, False)
+        self.assertEqual(request.code, 404)
+        self.assertEqual(request.getWrittenData(), 'Custom Not Found')
+        self.assertEqual(request.writeCount, 1)
 
     def test_requestWriteAfterFinish(self):
         app = self.app
@@ -764,17 +784,15 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(request.write.mock_calls, [call(''), call('foo')])
-            [failure] = self.flushLoggedErrors(RuntimeError)
+        self.assertFired(d)
+        self.assertEqual(request.writeCount, 2)
+        self.assertEqual(request.getWrittenData(), '')
+        [failure] = self.flushLoggedErrors(RuntimeError)
 
-            self.assertEqual(
-                str(failure.value),
-                ("Request.write called on a request after Request.finish was "
-                 "called."))
-
-        d.addCallback(_cb)
-        return d
+        self.assertEqual(
+            str(failure.value),
+            ("Request.write called on a request after Request.finish was "
+             "called."))
 
     def test_requestFinishAfterConnectionLost(self):
         app = self.app
@@ -800,9 +818,12 @@ class KleinResourceTests(unittest.TestCase):
         d.addErrback(lambda _: finished)
         d.addErrback(_eb)
 
+        self.assertNotFired(d)
+
         request.connectionLost(ConnectionLost())
 
-        return d
+        self.assertFired(d)
+
 
     def test_routeHandlesRequestFinished(self):
         app = self.app
@@ -821,13 +842,12 @@ class KleinResourceTests(unittest.TestCase):
 
         request.finish()
 
-        def _cb(result):
-            cancelled[0].trap(CancelledError)
-            request.write.assert_called_once_with('')
-            self.assertEqual(request.processingFailed.call_count, 0)
+        self.assertFired(d)
 
-        d.addCallback(_cb)
-        return d
+        cancelled[0].trap(CancelledError)
+        self.assertEqual(request.getWrittenData(), '')
+        self.assertEqual(request.writeCount, 1)
+        self.assertEqual(request.processingFailed.call_count, 0)
 
     def test_url_for(self):
         app = self.app
@@ -842,11 +862,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(relative_url[0], '/foo/2')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(relative_url[0], '/foo/2')
 
     def test_cancelledDeferred(self):
         app = self.app
@@ -862,12 +879,8 @@ class KleinResourceTests(unittest.TestCase):
 
         inner_d.cancel()
 
-        def _cb(result):
-            self.assertIdentical(result, None)
-            self.flushLoggedErrors(CancelledError)
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.flushLoggedErrors(CancelledError)
 
     def test_external_url_for(self):
         app = self.app
@@ -878,15 +891,13 @@ class KleinResourceTests(unittest.TestCase):
         @app.route("/foo/<int:bar>")
         def foo(request, bar):
             krequest = IKleinRequest(request)
-            relative_url[0] = krequest.url_for('foo', {'bar': bar + 1}, force_external=True)
+            relative_url[0] = krequest.url_for('foo', {'bar': bar + 1},
+                force_external=True)
 
         d = _render(self.kr, request)
 
-        def _cb(result):
-            self.assertEqual(relative_url[0], 'http://localhost:8080/foo/2')
-
-        d.addCallback(_cb)
-        return d
+        self.assertFired(d)
+        self.assertEqual(relative_url[0], 'http://localhost:8080/foo/2')
 
     def test_cancelledIsEatenOnConnectionLost(self):
         app = self.app
@@ -900,6 +911,8 @@ class KleinResourceTests(unittest.TestCase):
 
         d = _render(self.kr, request)
 
+        self.assertNotFired(d)
+
         request.connectionLost(ConnectionLost())
 
         def _cb(result):
@@ -907,7 +920,7 @@ class KleinResourceTests(unittest.TestCase):
 
         d.addErrback(lambda f: f.trap(ConnectionLost))
         d.addCallback(_cb)
-        return d
+        self.assertFired(d)
 
     def test_cancelsOnConnectionLost(self):
         app = self.app
@@ -920,14 +933,16 @@ class KleinResourceTests(unittest.TestCase):
             return handler_d
 
         d = _render(self.kr, request)
+
+        self.assertNotFired(d)
+
         request.connectionLost(ConnectionLost())
 
         handler_d.addErrback(lambda f: f.trap(CancelledError))
 
         d.addErrback(lambda f: f.trap(ConnectionLost))
         d.addCallback(lambda _: handler_d)
-
-        return d
+        self.assertFired(d)
 
     def test_ensure_utf8_bytes(self):
         self.assertEqual(ensure_utf8_bytes(u"abc"), "abc")
